@@ -17,6 +17,8 @@
 
 __all__ = [
     'TestDBus',
+    'TestDBusClient',
+    'TestDBusMain',
     'TestDBusMocksNoUpdate',
     'TestDBusMocksUpdateAvailable',
     'TestDBusMocksUpdateFailed',
@@ -25,12 +27,16 @@ __all__ = [
 
 import os
 import dbus
+import time
+import shutil
 import unittest
 
 from contextlib import ExitStack
+from datetime import datetime, timedelta
 from dbus.exceptions import DBusException
 from dbus.mainloop.glib import DBusGMainLoop
 from gi.repository import GLib
+from systemimage.bindings import DBusClient
 from systemimage.config import Configuration
 from systemimage.helpers import safe_remove
 from systemimage.testing.controller import Controller
@@ -39,31 +45,100 @@ from systemimage.testing.helpers import (
     sign)
 
 
-# 2013-07-25 BAW: This is an ugly hack caused by a weird problem I have not
-# been able to track down.  LP: #1205163
-_WHICH = 1
+_stack = None
+_controller = None
+
+# Why are these tests set up this?
+#
+# LP: #1205163 provides the impetus.  Here's the problem: we have to start a
+# dbus-daemon child process which will create an isolated system bus on which
+# our com.canonical.SystemImage service will be started via dbus-activatiion.
+# This closely mimics how the real system starts up our service.
+#
+# We ask dbus-daemon to return us its pid and the dbus address it's listening
+# on.  We need the address because we have to ensure that the dbus client,
+# i.e. this foreground test process, can communicate with the isolated
+# service.  To do this, the foreground process sets the environment variable
+# DBUS_SYSTEM_BUS_ADDRESS to the address that dbus-daemon gave us.
+#
+# The problem is that the low-level dbus client library only consults that
+# envar when it initializes, which it only does once per process.  There's no
+# way to get the library to listen on a new DBUS_SYSTEM_BUS_ADDRESS later on.
+#
+# This means that our first approach, which involved killing the granchild
+# system-image-dbus process, and the child dbus-daemon process, and then
+# restarting a new dbus-daemon process on a new address, doesn't work.
+#
+# We need a new system-image-dbus process for each of the TestCases below
+# because we have to start them up in different testing modes, and there's no
+# way to do that without exiting them and restarting them.  The grandchild
+# processes get started via different com.canonical.SystemImage.service files
+# with different commands.
+#
+# So, we have to restart the system-image-dbus process, but *not* the
+# dbus-daemon process because for all of these tests, it must be listening on
+# the same system bus.  Fortunately, dbus-daemon responds to SIGHUP, which
+# tells it to re-read its configuration files, including its .service files.
+# So how this works is that at the end of each test class, we tell the dbus
+# service to .Exit(), wait until it has, then write a new .service file with
+# the new command, HUP the dbus-daemon, and now the next time it activates the
+# service, it will do so with the correct (i.e. newly written) command.
+
+
+def setUpModule():
+    global _controller, _stack
+    _stack = ExitStack()
+    _controller = Controller()
+    _stack.callback(_controller.shutdown)
+    DBusGMainLoop(set_as_default=True)
+
+
+def tearDownModule():
+    global _controller, _stack
+    _stack.close()
+    _controller = None
 
 
 class _TestBase(unittest.TestCase):
     """Base class for all DBus testing."""
 
     # Override this to start the DBus server in a different testing mode.
-    mode = 'live'
+    mode = None
 
     @classmethod
     def setUpClass(cls):
-        cls._stack = ExitStack()
-        cls._controller = Controller(cls.mode)
-        cls._stack.callback(cls._controller.shutdown)
-        cls._controller.start()
-        DBusGMainLoop(set_as_default=True)
+        super().setUpClass()
+        _controller.set_testing_mode(cls.mode)
+        _controller.start()
 
     @classmethod
     def tearDownClass(cls):
-        cls._stack.close()
-        cls._controller = None
+        bus = dbus.SystemBus()
+        service = bus.get_object('com.canonical.SystemImage', '/Service')
+        iface = dbus.Interface(service, 'com.canonical.SystemImage')
+        iface.Exit()
+        # 2013-07-30 BAW: This sucks but there's no way to know exactly when
+        # the process has exited, because we cannot know the pid of the
+        # system-image-dbus process, which is our grandchild.  Just keep
+        # pinging the server until it stops responding.
+        until = datetime.now() + timedelta(seconds=60)
+        while datetime.now() < until:
+            time.sleep(0.2)
+            try:
+                iface.Exit()
+            except DBusException:
+                break
+        # Clear out the temporary directory.
+        config = Configuration()
+        config.load(_controller.ini_path)
+        try:
+            shutil.rmtree(config.system.tempdir)
+        except FileNotFoundError:
+            pass
+        super().tearDownClass()
 
     def setUp(self):
+        super().setUp()
         self.system_bus = dbus.SystemBus()
         service = self.system_bus.get_object(
             'com.canonical.SystemImage', '/Service')
@@ -71,6 +146,7 @@ class _TestBase(unittest.TestCase):
 
     def tearDown(self):
         self.iface.Reset()
+        super().tearDown()
 
     def _run_loop(self, method, signal):
         loop = GLib.MainLoop()
@@ -88,53 +164,63 @@ class _TestBase(unittest.TestCase):
         return signals
 
 
-@unittest.skipUnless(_WHICH == 1, 'TEST 1 - LP: #1205163')
-class TestDBus(_TestBase):
-    """Test the SystemImage dbus service."""
+class _LiveTesting(_TestBase):
+    mode = 'live'
 
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
+        cls._resources = ExitStack()
         # Set up the http/https servers that the dbus client will talk to.
         # Start up both an HTTPS and HTTP server.  The data files are vended
         # over the latter, everything else, over the former.
-        serverdir = cls._controller.serverdir
-        cls._stack.push(make_http_server(
-            serverdir, 8943, 'cert.pem', 'key.pem'))
-        cls._stack.push(make_http_server(serverdir, 8980))
-        # Set up the server files.
-        copy('channels_06.json', serverdir, 'channels.json')
-        sign(os.path.join(serverdir, 'channels.json'), 'image-signing.gpg')
-        # Only the archive-master key is pre-loaded.  All the other keys are
-        # downloaded and there will be both a blacklist and device keyring.
-        # The four signed keyring tar.xz files and their signatures end up in
-        # the proper location after the state machine runs to completion.
-        config = Configuration()
-        config.load(cls._controller.ini_path)
-        setup_keyrings('archive-master', use_config=config)
-        setup_keyring_txz(
-            'spare.gpg', 'image-master.gpg', dict(type='blacklist'),
-            os.path.join(serverdir, 'gpg', 'blacklist.tar.xz'))
-        setup_keyring_txz(
-            'image-master.gpg', 'archive-master.gpg',
-            dict(type='image-master'),
-            os.path.join(serverdir, 'gpg', 'image-master.tar.xz'))
-        setup_keyring_txz(
-            'image-signing.gpg', 'image-master.gpg',
-            dict(type='image-signing'),
-            os.path.join(serverdir, 'gpg', 'image-signing.tar.xz'))
-        setup_keyring_txz(
-            'device-signing.gpg', 'image-signing.gpg',
-            dict(type='device-signing'),
-            os.path.join(serverdir, 'stable', 'nexus7',
-                         'device-signing.tar.xz'))
+        serverdir = _controller.serverdir
+        try:
+            cls._resources.push(
+                make_http_server(serverdir, 8943, 'cert.pem', 'key.pem'))
+            cls._resources.push(make_http_server(serverdir, 8980))
+            # Set up the server files.
+            copy('channels_06.json', serverdir, 'channels.json')
+            sign(os.path.join(serverdir, 'channels.json'), 'image-signing.gpg')
+            # Only the archive-master key is pre-loaded.  All the other keys
+            # are downloaded and there will be both a blacklist and device
+            # keyring.  The four signed keyring tar.xz files and their
+            # signatures end up in the proper location after the state machine
+            # runs to completion.
+            config = Configuration()
+            config.load(_controller.ini_path)
+            setup_keyrings('archive-master', use_config=config)
+            setup_keyring_txz(
+                'spare.gpg', 'image-master.gpg', dict(type='blacklist'),
+                os.path.join(serverdir, 'gpg', 'blacklist.tar.xz'))
+            setup_keyring_txz(
+                'image-master.gpg', 'archive-master.gpg',
+                dict(type='image-master'),
+                os.path.join(serverdir, 'gpg', 'image-master.tar.xz'))
+            setup_keyring_txz(
+                'image-signing.gpg', 'image-master.gpg',
+                dict(type='image-signing'),
+                os.path.join(serverdir, 'gpg', 'image-signing.tar.xz'))
+            setup_keyring_txz(
+                'device-signing.gpg', 'image-signing.gpg',
+                dict(type='device-signing'),
+                os.path.join(serverdir, 'stable', 'nexus7',
+                             'device-signing.tar.xz'))
+        except:
+            cls._resources.close()
+            raise
+
+    @classmethod
+    def tearDownClass(cls):
+        cls._resources.close()
+        super().tearDownClass()
 
     def setUp(self):
         super().setUp()
         self._prepare_index('index_13.json')
         # We need a configuration file that agrees with the dbus client.
         self.config = Configuration()
-        self.config.load(self._controller.ini_path)
+        self.config.load(_controller.ini_path)
         # For testing reboot preparation.
         self.command_file = os.path.join(
             self.config.updater.cache_partition, 'ubuntu_command')
@@ -150,17 +236,20 @@ class TestDBus(_TestBase):
 
     def _prepare_index(self, index_file):
         index_path = os.path.join(
-            self._controller.serverdir, 'stable', 'nexus7', 'index.json')
+            _controller.serverdir, 'stable', 'nexus7', 'index.json')
         head, tail = os.path.split(index_path)
         copy(index_file, head, tail)
         sign(index_path, 'device-signing.gpg')
-        setup_index(
-            index_file, self._controller.serverdir, 'device-signing.gpg')
+        setup_index(index_file, _controller.serverdir, 'device-signing.gpg')
 
     def _set_build(self, version):
         with open(self.config.system.build_file, 'w', encoding='utf-8') as fp:
             print(version, file=fp)
 
+
+class TestDBus(_LiveTesting):
+    """Test the SystemImage dbus service."""
+
     def test_check_build_number(self):
         # Get the build number.
         self._set_build(20130701)
@@ -295,7 +384,7 @@ unmount system
         self.iface.Reboot()
         with open(self.reboot_log, encoding='utf-8') as fp:
             reboot = fp.read()
-        self.assertEqual(reboot, 'reboot -f recovery')
+        self.assertEqual(reboot, '/sbin/reboot -f recovery')
 
     def test_reboot_no_update(self):
         # There's no update to reboot to.
@@ -314,7 +403,7 @@ unmount system
         # A signal is issued when the update failed.
         #
         # Cause the update to fail by deleting a file from the server.
-        os.remove(os.path.join(self._controller.serverdir, '4/5/6.txt.asc'))
+        os.remove(os.path.join(_controller.serverdir, '4/5/6.txt.asc'))
         signals = self._run_loop(self.iface.GetUpdate, 'UpdateFailed')
         self.assertEqual(len(signals), 1)
 
@@ -322,7 +411,7 @@ unmount system
         # Cause the update to fail by deleting a file from the server.
         #
         # Cause the update to fail by deleting a file from the server.
-        os.remove(os.path.join(self._controller.serverdir, '4/5/6.txt.asc'))
+        os.remove(os.path.join(_controller.serverdir, '4/5/6.txt.asc'))
         signals = self._run_loop(self.iface.GetUpdate, 'UpdateFailed')
         self.assertEqual(len(signals), 1)
         signals = self._run_loop(self.iface.Reboot, 'UpdateFailed')
@@ -354,7 +443,6 @@ unmount system
         self.assertEqual(self.iface.BuildNumber(), 0)
 
 
-@unittest.skipUnless(_WHICH == 2, 'TEST 2 - LP: #1205163')
 class TestDBusMocksNoUpdate(_TestBase):
     mode = 'no-update'
 
@@ -369,14 +457,13 @@ class TestDBusMocksNoUpdate(_TestBase):
         self.assertFalse(signals[0][0])
 
 
-@unittest.skipUnless(_WHICH == 3, 'TEST 3 - LP: #1205163')
 class TestDBusMocksUpdateAvailable(_TestBase):
     mode = 'update-success'
 
     def setUp(self):
         super().setUp()
         config = Configuration()
-        config.load(self._controller.ini_path)
+        config.load(_controller.ini_path)
         self.reboot_log = os.path.join(
             config.updater.cache_partition, 'reboot.log')
 
@@ -430,7 +517,7 @@ class TestDBusMocksUpdateAvailable(_TestBase):
         self.iface.Reboot()
         with open(self.reboot_log, encoding='utf-8') as fp:
             reboot = fp.read()
-        self.assertEqual(reboot, 'reboot -f recovery')
+        self.assertEqual(reboot, '/sbin/reboot -f recovery')
 
     def test_reboot_canceled(self):
         self._run_loop(self.iface.CheckForUpdate, 'UpdateAvailableStatus')
@@ -443,7 +530,6 @@ class TestDBusMocksUpdateAvailable(_TestBase):
         self.assertFalse(os.path.exists(self.reboot_log))
 
 
-@unittest.skipUnless(_WHICH == 4, 'TEST 4 - LP: #1205163')
 class TestDBusMocksUpdateFailed(_TestBase):
     mode = 'update-failed'
 
@@ -482,3 +568,105 @@ class TestDBusMocksUpdateFailed(_TestBase):
         self.assertEqual(len(signals), 1)
         signals = self._run_loop(self.iface.Reboot, 'UpdateFailed')
         self.assertEqual(len(signals), 1)
+
+
+class TestDBusMain(_TestBase):
+    mode = 'live'
+
+    def setUp(self):
+        # Don't call super's setUp() since that will start the service, thus
+        # creating the temporary directory.
+        pass
+
+    def tearDown(self):
+        # We didn't call setUp() so don't call tearDown().
+        pass
+
+    def test_temp_directory(self):
+        # The temporary directory gets created if it doesn't exist.
+        config = Configuration()
+        config.load(_controller.ini_path)
+        self.assertFalse(os.path.exists(config.system.tempdir))
+        # DBus activate the service, which should create the directory.
+        bus = dbus.SystemBus()
+        service = bus.get_object('com.canonical.SystemImage', '/Service')
+        self.assertTrue(os.path.exists(config.system.tempdir))
+
+
+class TestDBusClient(_LiveTesting):
+    """Test the DBus client (used with --dbus)."""
+
+    def setUp(self):
+        super().setUp()
+        self._client = DBusClient()
+
+    def test_build_number(self):
+        # Get the build number through the client.
+        self._set_build(20130701)
+        self.assertEqual(self._client.build_number, 20130701)
+
+    def test_check_for_update(self):
+        # There is an update available.
+        self.assertTrue(self._client.check_for_update())
+
+    def test_check_for_no_update(self):
+        # There is no update available.
+        self._set_build(20130701)
+        self.assertFalse(self._client.check_for_update())
+
+    def test_get_update_size(self):
+        # The size of the available update.
+        self.assertTrue(self._client.check_for_update())
+        self.assertEqual(self._client.update_size, 314572800)
+
+    def test_get_update_no_size(self):
+        # Size is zero if there is no update available.
+        self._set_build(20130701)
+        self.assertFalse(self._client.check_for_update())
+        self.assertEqual(self._client.update_size, 0)
+
+    def test_get_update_version(self):
+        # The target version of the upgrade.
+        self.assertTrue(self._client.check_for_update())
+        self.assertEqual(self._client.update_version, 20130600)
+
+    def test_get_update_no_version(self):
+        # Version is zero if there is no update available.
+        self._set_build(20130701)
+        self.assertFalse(self._client.check_for_update())
+        self.assertEqual(self._client.update_version, 0)
+
+    def test_get_update_descriptions(self):
+        # The update has some descriptions.
+        self.assertTrue(self._client.check_for_update())
+        self.assertEqual(self._client.update_descriptions,
+                         [{'description': 'Full'}])
+
+    def test_get_update_no_descriptions(self):
+        # Sometimes there's no update, and thus no descrptions.
+        self._set_build(20130701)
+        self.assertFalse(self._client.check_for_update())
+        self.assertEqual(self._client.update_descriptions, [])
+
+    def test_update(self):
+        # Do the update, but wait to reboot.
+        self.assertTrue(self._client.check_for_update())
+        ready = self._client.update()
+        self.assertTrue(ready)
+
+    def test_update_failed(self):
+        # For some reason <wink>, the update fails.
+        self.assertTrue(self._client.check_for_update())
+        # Cause the update to fail by deleting a file from the server.
+        os.remove(os.path.join(_controller.serverdir, '4/5/6.txt.asc'))
+        ready = self._client.update()
+        self.assertFalse(ready)
+
+    def test_reboot(self):
+        # After a successful update, we can reboot.
+        self.assertTrue(self._client.check_for_update())
+        self.assertTrue(self._client.update())
+        self._client.reboot()
+        with open(self.reboot_log, encoding='utf-8') as fp:
+            reboot = fp.read()
+        self.assertEqual(reboot, '/sbin/reboot -f recovery')
