@@ -61,6 +61,9 @@ class TestCLIMain(unittest.TestCase):
                 resource_filename('systemimage.data', 'client.ini'), tempdir)
             stack.enter_context(
                 patch('systemimage.main.DEFAULT_CONFIG_FILE', ini_path))
+            # Mock out the initialize() call so that the main() doesn't try to
+            # create a log file in a non-existent system directory.
+            stack.enter_context(patch('systemimage.main.initialize'))
             cli_main()
             self.assertEqual(config.config_file, ini_path)
             self.assertEqual(config.system.build_file, '/etc/ubuntu-build')
@@ -81,11 +84,9 @@ class TestCLIMain(unittest.TestCase):
             with self.assertRaises(SystemExit) as cm:
                 cli_main()
             self.assertEqual(cm.exception.code, 2)
-            self.assertEqual(stderr.getvalue(), """\
-usage: system-image-cli [-h] [--version] [-C FILE] [-b] [-c] [-u NUMBER] [-v]
-system-image-cli: error:\x20
-Configuration file not found: /does/not/exist/client.ini
-""")
+            self.assertEqual(
+                stderr.getvalue().splitlines()[-1],
+                'Configuration file not found: /does/not/exist/client.ini')
 
     def test_missing_explicit_config_file(self):
         # An explicit configuration file given with -C is missing.
@@ -100,11 +101,9 @@ Configuration file not found: /does/not/exist/client.ini
             with self.assertRaises(SystemExit) as cm:
                 cli_main()
             self.assertEqual(cm.exception.code, 2)
-            self.assertEqual(stderr.getvalue(), """\
-usage: system-image-cli [-h] [--version] [-C FILE] [-b] [-c] [-u NUMBER] [-v]
-system-image-cli: error:\x20
-Configuration file not found: /does/not/exist.ini
-""")
+            self.assertEqual(
+                stderr.getvalue().splitlines()[-1],
+                'Configuration file not found: /does/not/exist.ini')
 
     def test_ensure_directories_exist(self):
         # The temporary and var directories are created if they don't exist.
@@ -176,6 +175,29 @@ Configuration file not found: /does/not/exist.ini
             self.assertEqual(capture.getvalue(),
                              'channel/device: stable/nexus7\n')
 
+    @testable_configuration
+    def test_log_file(self, ini_file):
+        # Test that the system log file gets created and written.
+        config = Configuration()
+        config.load(ini_file)
+        self.assertFalse(os.path.exists(config.system.logfile))
+        class FakeState:
+            def __iter__(self):
+                return self
+            def __next__(self):
+                raise StopIteration
+        with ExitStack() as stack:
+            stack.enter_context(patch('systemimage.main.sys.argv',
+                                      ['argv0', '-C', ini_file]))
+            stack.enter_context(patch('systemimage.main.State', FakeState))
+            cli_main()
+        self.assertTrue(os.path.exists(config.system.logfile))
+        with open(config.system.logfile, encoding='utf-8') as fp:
+            logged = fp.read()
+        # Ignore any leading timestamp and the trailing newline.
+        self.assertEqual(logged[-38:-1],
+                         'running state machine [stable/nexus7]')
+
 
 @unittest.skip('dbus-launch only supports session bus (LP: #1206588)')
 @unittest.skipUnless(DBUS_LAUNCH is not None, 'dbus-launch not found')
