@@ -32,7 +32,6 @@ from contextlib import ExitStack
 from subprocess import CalledProcessError
 from systemimage.config import config
 from systemimage.gpg import SignatureError
-from systemimage.logging import initialize
 from systemimage.state import State
 from systemimage.testing.helpers import (
     copy, get_index, make_http_server, setup_index, setup_keyring_txz,
@@ -40,13 +39,6 @@ from systemimage.testing.helpers import (
 from unittest.mock import patch
 
 
-def setUpModule():
-    # BAW 2013-06-17: For correctness, this really should be put in all
-    # test_*.py modules, or in a global test runner.  As it is, this only
-    # quiets the logging output for tests in this module and later.
-    initialize(verbosity=3)
-
-
 class TestState(unittest.TestCase):
     """Test various state transitions."""
 
@@ -438,7 +430,8 @@ class TestRebooting(_StateTestsBase):
         self._setup_keyrings()
         with patch('systemimage.reboot.check_call') as mock:
             list(State())
-        self.assertEqual(mock.call_args[0][0], ['reboot', '-f', 'recovery'])
+        self.assertEqual(mock.call_args[0][0],
+                         ['/sbin/reboot', '-f', 'recovery'])
 
 
     @testable_configuration
