@@ -29,7 +29,6 @@ from systemimage.config import config
 from systemimage.download import get_files
 from systemimage.gpg import SignatureError
 from systemimage.helpers import temporary_directory
-from systemimage.logging import initialize
 from systemimage.scores import WeightedScorer
 from systemimage.state import ChecksumError, State
 from systemimage.testing.helpers import (
@@ -38,13 +37,6 @@ from systemimage.testing.helpers import (
     testable_configuration)
 
 
-def setUpModule():
-    # BAW 2013-06-17: For correctness, this really should be put in all
-    # test_*.py modules, or in a global test runner.  As it is, this only
-    # quiets the logging output for tests in this module and later.
-    initialize(verbosity=3)
-
-
 class TestWinnerDownloads(unittest.TestCase):
     """Test full end-to-end downloads through index.json."""
 
